#! /usr/bin/Rscript

#' @title Run SIRIUS
#'
#' @description
#'
#' This function simple runs SIRIUS based on the argumenst given to the function 
#' 
#' 


#' @param files is the csv file that contains the input and outputs paths
#'
#' @param ppm_max the ppm range to be explored with the precursor masses found in the DBs
#'
#' @param ppm_max_ms2, the ppm range to be explored with the fragmenat peaks found in the DBs
#'
#' @param QC, whether isotopic peak information was used or not
#'
#' @param SL_path path to the suspect list fragmentation files generated by custom db, a SIRIUS function
#'
#' @param SL either TRUE or FALSE (depends is the user has used 
#' inhouse library or here called a suspect list)
#'
#' @param candidates no. of formula candidates considered
#'

#' @return
#' 
#' result .json directories for SIRIUS in a folder with the same name 
#' /insilico/SIRIUS/param1.json
#' 
#' Runs SIRIUS and generates json result directories for SIRIUS
#' 
#' @author Mahnoor Zulfiqar
#' 
#' @examples
#' 
#' sirius_param(x = /usr/project/file1/insilico/MS1DATA_SiriusPandSL.csv',
#'                      ppm_max = 5,
#'                      ppm_max_ms2 = 15,
#'                      QC = TRUE,
#'                      SL = TRUE,
#'                      SL_path = usr/project/ScostSLS/)


# ---------- Preparations ----------
# Load libraries


# ---------- Arguments and user variables ----------
args <- commandArgs(trailingOnly=TRUE)
#print(args)

files <- as.character(args[1])
ppm_max <- as.numeric(args[2])
ppm_max_ms2 <- as.numeric(args[3])
QC <- as.logical(args[4])
SL <- as.logical(args[5])
SL_path <- as.character(args[6])
candidates <- as.numeric(args[7])

# ---------- run_sirius ----------

run_sirius <- function(files, ppm_max = 5, ppm_max_ms2 = 15, QC = TRUE, SL = TRUE, SL_path, candidates = 30){
    
    files <- read.csv(files)
    
    for (b in 1:nrow(files)){
        if (QC){
            if (is.na(files[b, "isotopes"])){
                system(paste("sirius --input", files[b, "sirius_param_file"], "--output", files[b, "outputNames"],
                             "formula --profile orbitrap --no-isotope-filter --no-isotope-score --candidates", candidates, "--ppm-max", ppm_max, "--ppm-max-ms2",  ppm_max_ms2,"structure --database ALL canopus",
                             sep = " "))
                if(SL){
                    system(paste("sirius --input", files[b, "sirius_param_file"], "--output", files[b, "outputNamesSL"],
                             "formula --profile orbitrap --no-isotope-filter --no-isotope-score --candidates", candidates, "--ppm-max", ppm_max, "--ppm-max-ms2",  ppm_max_ms2,"structure --database",  SL_path ,"canopus",
                             sep = " "))
                }
                
            }
            else if(files[b, "isotopes"] == "present"){
                system(paste("sirius --input", files[b, "sirius_param_file"], "--output", files[b, "outputNames"],
                             "formula --profile orbitrap --candidates", candidates, "--ppm-max", ppm_max,"--ppm-max-ms2", ppm_max_ms2,"structure --database ALL canopus",
                             sep = " "))
                if(SL){
                    system(paste("sirius --input", files[b, "sirius_param_file"], "--output", files[b, "outputNamesSL"],
                             "formula --profile orbitrap --candidates", candidates, "--ppm-max", ppm_max, "--ppm-max-ms2",  ppm_max_ms2,"structure --database",  SL_path ,"canopus",
                             sep = " "))
                }
            }
        }
        else{
            system(paste("sirius --input", files[b, "sirius_param_file"], "--output", files[b, "outputNames"],
                             "formula --profile orbitrap --no-isotope-filter --no-isotope-score --candidates", candidates, "--ppm-max", ppm_max, "--ppm-max-ms2",  ppm_max_ms2,"structure --database ALL canopus",
                             sep = " "))
            if(SL){
                system(paste("sirius --input", files[b, "sirius_param_file"], "--output", files[b, "outputNamesSL"],
                            "formula --profile orbitrap --no-isotope-filter --no-isotope-score --candidates", candidates, "--ppm-max", ppm_max, "--ppm-max-ms2",  ppm_max_ms2,"structure --database",  SL_path ,"canopus",
                            sep = " "))
            }
        }
    }
}

run_sirius(files, ppm_max, ppm_max_ms2, QC, SL, SL_path, candidates)

